import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * A Phone Book that maps names to phone numbers.
 * 
 * @author Chuck Cusack
 * @version 2/23/24
 */
public class PhoneBook
{
    HashMap<String,String> phoneNumbers;
    
    public PhoneBook() {
        phoneNumbers = new HashMap<>();
    }
    
    /**
     * Add an phone book entry
     * @param name the name for the entry
     * @param the phone number for the entry.
     */
    public void insert(String name, String number) {
        phoneNumbers.put(name, number);
    }
    
    /**
     * Returns the phone number for name, or
     * "unlisted" if the name is not in the phone book.
     * @param name the name to look up
     * @return the phone number for name, or 
     * "unlisted" if the name is not in the book.
     */
    public String getNumber(String name) {
        if(phoneNumbers.containsKey(name)) {
            return phoneNumbers.get(name);
        } else {
            return "unlisted";
        }
    }
    
    /**
     * Returns the phone number for the person with name name.
     * @return the phone number for the person with name name.
     */
    public boolean numberListed(String name) {
        return phoneNumbers.containsKey(name);
    }
    
       
    /**
     * Returns the number of numbers in the phone book.
     * @return the number of numbers in the phone book.
     */
    public int numberOfListings() {
        return phoneNumbers.size(); 
    }
    
    /**
     * Print out the list of names and phone numbers, one per line.
     */
    public void printPhoneBook() {
        // Version 1: iterate using keySet().
        // Set<String> keys = phoneNumbers.keySet();
        // for(String name : keys) {
            // String number = phoneNumbers.get(name);
            // System.out.println(name+": "+number);
        // }
        
        // Version 2: Iterate using entrySet().
        // Set<Map.Entry<String,String>> entries;
        // entries = phoneNumbers.entrySet();
        // for(Map.Entry<String,String> entry : entries) {
            // System.out.println(entry.getKey()+": "+entry.getValue());
        // }
        
        // Version 3: Iterate using entrySet(), but the shorter version 
        // that doesn't use a local variable because it is really unnecessary.
        // Not that phoneMumbers().entrySet() returns an object of type
        // Set<Map.Entry<String,String>>.
        // Thus, when iterating over it in a for-each loop, the data type of
        // the entries is Map.Entry<String,String>.
        for(Map.Entry<String,String> entry : phoneNumbers.entrySet()) {
            System.out.println(entry.getKey()+": "+entry.getValue());
        }
        
    }
    
    /**
     * Returns the name of the person with the given number.
     * @param the number to look up.
     * @return the name associated with the number, or null if the
     *    number is not listed.
     */
    public String reverseLookUp(String number) {
        for(Map.Entry<String,String> entry : phoneNumbers.entrySet()) {
            if(entry.getValue().equals(number)) {
                return entry.getKey();
            }
        }
        return null;
    }
    
}
